
/*
  Program uChipTGPS.ino by G.Carrera for uChip rev. 17/04/2021
  this program uses the following components:
    1x uChip card,
    1x 2.4" touch TFT display (240x320 pixels)
    1x GNSS receiver
  
  The TFT library is written by Limor Fried/Ladyada for Adafruit Industries
  The XPT2046 touch library is written by Paul Stoffregen, paul@pjrc.com
  The uChip library is from www.itaca-innovation.com
  The TinyGPSPlus library is written by Mikal Hart, https://github.com/mikalhart/TinyGPSPlus
*/
//include library code
#include <Adafruit_GFX.h>
#include <Adafruit_ILI9341.h>
#include <SPI.h>
#include <SD.h>
#include <XPT2046_Touchscreen.h>
#include <uChipPowerManagement.h>
#include <TinyGPS++.h>
// The TinyGPS++ object
TinyGPSPlus gps;
TinyGPSCustom Qind(gps,"GPGGA",6); // extract Quality index (char)
TinyGPSCustom Times(gps,"GPGGA",1); // extract time (char array)
TinyGPSCustom Dates(gps,"GPRMC",9); // extract date (char array)
//***********************************************************************************
//define pins of TFT screen
#define TFT_CS     7
#define TFT_DC     12
#define TFTLed  15
Adafruit_ILI9341 tft = Adafruit_ILI9341(TFT_CS, TFT_DC);// hasware SPI
#define SD_CS  4 // CS SD card slot on display
// pin for touchscreen
#define T_CS  2 
#define T_IRQ  13
XPT2046_Touchscreen ts(T_CS, T_IRQ);// interrupt enabled polling
boolean SDok, ok;
boolean fixok = false;
boolean fixmean = false;
boolean Acqf = false;
boolean filnam = false; 
long vep = 0;
String hhmmss;
String filename;
const String c = ",";
double Latd, Longd, h;
float HDOP;
int nsat;
byte Qi = 0;
double Latm = 0;
double Longm = 0;
long BaudRate = 9600;
long Number;
int X,Y;
int nm = 0;
int ns =0;
int NuAv = 60; // number of averages
double radc = M_PI/180;// degree to radiant conversion
//***********************************************************************************
void setup(void) {
  uChipSetVextValue(VEXT_3V3);
  pinMode(TFTLed, OUTPUT);// TFT backlight (ON = LOW)
  ts.begin();// initialize XPT2046_Touchscreen
  if (!SD.begin(SD_CS)) {
    SDok = false;   
  } else SDok = true;
  tft.begin();
  digitalWrite(TFTLed, LOW);// turn ON display leds
  tft.fillScreen(ILI9341_BLACK);
  tft.setTextColor(ILI9341_CYAN);
  tft.setTextSize(2);
  tft.setCursor(0,50);
  tft.print("uChip GNSS logger");
  tft.setCursor(0,84);
  if (SDok) tft.print("SD initialised!");
  else {
    tft.setTextColor(ILI9341_RED);
    tft.print("SD card failed!");    
  }
  delay(2000);
  ok = false;
  Serial1.begin(BaudRate);// serial port for GPS interface
  tft.setCursor(0,124);
  tft.setTextColor(ILI9341_WHITE);
  tft.println("Waiting for valid");
  tft.print("GPS data .......");
} // end of setup

void loop() {
  while (Serial1.available() > 0){
    gps.encode(Serial1.read());
    if (gps.location.isUpdated() && Qind.isUpdated() && 
       gps.altitude.isUpdated() && gps.satellites.isUpdated() && 
       gps.hdop.isUpdated() && Times.isUpdated() && Dates.isUpdated()){   
      GetData();
    }
  }
}	
/************************* FUNCTIONS *************************/

void GetData() {
  String Qis = Qind.value();
  Qi = Qis.toInt();
  if (Qi > 0) fixok= true;
  else fixok= false;// not valid data
  Latd = gps.location.lat(); // Latitude in degrees (double)
  Longd = gps.location.lng(); // Longitude in degrees (double)
  h = gps.altitude.meters(); // Altitude in meters (double)
  hhmmss = Times.value(); // time in HHMMSS.CC String
  hhmmss = hhmmss.substring(0,6);// only hours, minutes and seconds
  HDOP = gps.hdop.value()/100.0; // Horizontal Dilution of Precision
  nsat = gps.satellites.value(); // Number of satellites in use
  if (SDok && fixok) {// generate the file name based on date and hour
    String hhs = hhmmss.substring(0,2);// only hours
    String ddmmdd = Dates.value(); 
    filename = ddmmdd+hhs+".CSV";// the csv file name is the date and hour
  }
  if (nm < NuAv && fixok){// calculates the average of the initial fix
    Latm += Latd;
    Longm += Longd;
    nm++;
    if (nm == NuAv){
       fixmean= true;
       Latm /= NuAv;
       Longm /= NuAv;  
    }
  }  
  if (SDok && fixok && Acqf) {
    File dataFile = SD.open(filename, FILE_WRITE);            
    if (dataFile) {  // if the file is available, write to it:
      dataFile.print(Latd,7);
      dataFile.print(c);
      dataFile.print(Longd,7);
	    String outdata = c+hhmmss+c+String(nsat)+c+String(HDOP,1)+c+String(h,1); // other data to record
      dataFile.println(outdata);
      vep++;
      dataFile.close();
    }
   }
   DisplayData();
}

void DisplayData(){
  int yline = 0;
  tft.setTextSize(2);
  tft.fillScreen(ILI9341_BLACK);
  printline("Time= " + hhmmss,ILI9341_WHITE,yline); yline+= 20;
  printline("Lat dec= " + String(Latd,7),ILI9341_WHITE,yline); yline+= 20;
  printline("Long dec= " + String(Longd,7),ILI9341_WHITE,yline); yline+= 20;
  printline("N. satellites = "+ String(nsat),ILI9341_WHITE,yline); yline+= 20;;
  printline("HDOP= "+String(HDOP,1),ILI9341_WHITE,yline); yline+= 20;;
  printline("Quality F= "+String(Qi),ILI9341_WHITE,yline); yline+= 20;
  printline("Altitude= "+String(h,1)+" [m]",ILI9341_WHITE,yline); yline+= 20;
  if (fixmean){
    printline("Lat me= " + String(Latm,7),ILI9341_WHITE,yline); yline+= 20;
    printline("Long me= " + String(Longm,7),ILI9341_WHITE,yline); yline+= 20;
	// calculates the distance from the starting point
    double Dist = gps.distanceBetween(Latm,Longm,Latd,Longd);	
    printline("Distance= "+ String(Dist,1),ILI9341_CYAN,yline); yline+= 20;
	// calculates the course to the starting point
    float Course = gps.courseTo(Latd,Longd,Latm,Longm);//  course to start point
    printline("Course= "+ String(Course,1),ILI9341_CYAN,yline);yline+= 20;
  } else {
    printline("N means = " + String(nm),ILI9341_YELLOW,yline); yline+= 20;
  }
  if (Acqf){
    ns++;
    printline("N samples = " + String(ns),ILI9341_YELLOW,yline); yline+= 20;
  }
  AcqStart_Stop();
}

void printline(String text, uint16_t color, uint8_t ypos ) {
  tft.setCursor(0, ypos);
  tft.setTextColor(color);
  tft.setTextWrap(true);
  tft.print(text);
}

void AcqStart_Stop() {
  tft.fillRoundRect(0,260,240,60,15,ILI9341_BLUE);// ACQ START/STOP
  tft.drawRoundRect(0,260,240,60,15,ILI9341_BLACK);
  tft.setTextColor(ILI9341_YELLOW);
  tft.setCursor(20, 284);
  tft.setTextSize(2);
  if (!Acqf && SDok){
    tft.println("Acq. Start/Stop");    
  } else if (SDok) tft.println("Acquisizione...");
  DetectSS();
}
void DetectSS(){
    if (ts.touched()) {
      TS_Point p = ts.getPoint();
      X= p.y; Y= p.x;// invert the axes
      if (Y>3100 && Y<3780){// ACQ START/STOP
         if (!Acqf && SDok){
           Acqf = true;
           tft.fillRoundRect(0,260,240,59,15,ILI9341_RED);// ACQ START/STOP
           tft.setTextColor(ILI9341_WHITE);
           tft.setCursor(10, 284);
           tft.setTextSize(2);
           tft.println("Inizio Acquisizione");
         } else if (SDok){
           Acqf = false;
           tft.fillRoundRect(0,260,240,59,15,ILI9341_GREEN);// ACQ START/STOP
           tft.setTextColor(ILI9341_BLUE);
           tft.setCursor(20, 284);
           tft.setTextSize(2);
           tft.println("Fine Acquisizione");          
           ok = true;
         }
         delay(1000);          
      }
    }  
}
